class ActivityAction:
    @staticmethod
    def create_activity():
        raise NotImplementedError

    @staticmethod
    def add_activity():
        raise NotImplementedError

    @staticmethod
    def list_activities():
        raise NotImplementedError

    @staticmethod
    def modify_activity():
        raise NotImplementedError

    @staticmethod
    def delete_activity():
        raise NotImplementedError

    @staticmethod
    def export_activities():
        raise NotImplementedError
